package com.contextspace;

import static org.junit.Assert.*;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.jdo.annotations.Index;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Persistence;
import javax.persistence.TableGenerator;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.TypedQuery;

import org.junit.Test;

public class TestSearch {

    @Test
    public void test() {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("objectdb:$objectdb/db/test.tmp;drop");
        EntityManager em = emf.createEntityManager();
     
        
        em.getTransaction().begin();
        
        for(int i=0; i<100; i++) {
            CDI cdi1 = new CDI("first" + i + "_last" + i,"first" + i, "last" + 1, new Date());
            em.persist(cdi1);
        }
        em.getTransaction().commit();
     
        TypedQuery<Long> query = em.createQuery("SELECT COUNT($1) FROM CDI $1 WHERE $1.loginId LIKE '%' ESCAPE '\\'", Long.class);
        Long cdiCount = query.getSingleResult();
        assertEquals(100, cdiCount.intValue());
        
        TypedQuery<Long> query2 = em.createQuery("SELECT COUNT($1) FROM CDI $1 WHERE $1.loginId LIKE 'first%' ESCAPE '\\'", Long.class);
        Long cdiCount2 = query2.getSingleResult();
        assertEquals(100, cdiCount2.intValue());
     
        TypedQuery<Long> query3 = em.createQuery("SELECT COUNT($1) FROM CDI $1 WHERE $1.loginId LIKE 'first%\\_%' ESCAPE '\\'", Long.class);
        Long cdiCount3 = query3.getSingleResult();
        assertEquals(100, cdiCount3.intValue());
        
        TypedQuery<Long> query4 = em.createQuery("SELECT COUNT($1) FROM CDI $1 WHERE $1.loginId LIKE 'first1\\_%' ESCAPE '\\'", Long.class);
        Long cdiCount4 = query4.getSingleResult();
        assertEquals(1, cdiCount4.intValue());
        
        TypedQuery<CDI> query5 = em.createQuery("SELECT $1 FROM CDI $1 WHERE $1.loginId LIKE 'first1*_%' ESCAPE '*'", CDI.class);
        List<CDI> cdiCount5 = query5.getResultList();
        assertEquals(1, cdiCount5.size());
        System.out.println("TEST5: " + cdiCount5.get(0).getLoginId());
        assertEquals("first1_last1", cdiCount5.get(0).getLoginId());
        
        TypedQuery<CDI> query6 = em.createQuery("SELECT $1 FROM CDI $1 WHERE $1.loginId LIKE 'first%*_%2' ESCAPE '*'", CDI.class);
        List<CDI> cdiCount6 = query6.getResultList();
        //assertEquals(1, cdiCount6.size());
        for(CDI cdi : cdiCount6) {
            System.out.println("TEST6: " + cdi.getLoginId());
        }
        
        TypedQuery<CDI> query7 = em.createQuery("SELECT $1 FROM CDI $1 WHERE $1.loginId LIKE '%*_last%2' ESCAPE '*'", CDI.class);
        List<CDI> cdiCount7 = query7.getResultList();
        //assertEquals(1, cdiCount6.size());
        for(CDI cdi : cdiCount7) {
            System.out.println("TEST7: " + cdi.getLoginId());
        }
        
        em.close();
        emf.close();
    }

    
    @Entity
    @TableGenerator(name="IdTableGen", initialValue=1, allocationSize=50)
    public static class CDI implements Serializable{
        private static final long serialVersionUID = 1L;
       
        private Date creationDate = null;
        private String loginId = null;
        private String firstname = null;
        private String lastname = null;
        
        private long id;
        
        /**
         * Default Constructor
         */
        public CDI() {
            
        }
        
        /**
         * Constructor
         * @param firstname
         * @param lastname
         */
        public CDI(String firstname, String lastname) {
            this.firstname = firstname;
            this.lastname = lastname;
            this.creationDate = new Date();
        }
        
        /**
         * Constructor
         * @param firstname
         * @param lastname
         * @param creationDate
         */
        public CDI(String loginId, String firstname, String lastname, Date creationDate) {
            this.loginId = loginId;
            this.firstname = firstname;
            this.lastname = lastname;
            this.creationDate = creationDate;
        }
            
        /**
         * @return the creationDate
         */
        @Temporal(TemporalType.TIMESTAMP)
        public Date getCreationDate() {
            return creationDate;
        }
        
        /**
         * @param creationDate the creationDate to set
         */
        public void setCreationDate(Date creationDate) {
            this.creationDate = creationDate;
        }
        
        /**
         * @return the object id
         */
        @Id
        @GeneratedValue(strategy=GenerationType.TABLE, generator="IdTableGen")
        public long getId() {
            return id;
        }

        /**
         * @param id the id to be set
         */
        public void setId(long id) {
            this.id = id;
        }

        /**
         * @return the firstname
         */
        @Index(unique="false")
        public String getFirstname() {
            return firstname;
        }

        /**
         * @param firstname the firstname to set
         */
        public void setFirstname(String firstname) {
            this.firstname = firstname;
        }

        /**
         * @return the lastname
         */
        @Index(unique="false")
        public String getLastname() {
            return lastname;
        }

        /**
         * @param lastname the lastname to set
         */
        public void setLastname(String lastname) {
            this.lastname = lastname;
        }

        /**
         * @return the loginId
         */
        @Index(unique="true")
        @Column(nullable=false,unique=true)
        public String getLoginId() {
            return loginId;
        }

        /**
         * @param loginId the loginId to set
         */
        public void setLoginId(String loginId) {
            this.loginId = loginId;
        }
        
    }
}
